{-# LANGUAGE LambdaCase      #-}
{-# OPTIONS_GHC -Wno-orphans #-}

module Main where

import Data.Aeson.Encode.Pretty
import Data.ByteString qualified as B
import Data.ByteString.Lazy qualified as BL
import Gargantext.API.Routes.Named
import Options.Applicative
import Prelude
import Servant.API
import Servant.API.Routes
import Servant.Auth qualified as Servant

data CLIRoutes
  = CLIR_list
  | CLIR_export FilePath
  deriving (Show, Eq)

data CLI =
  CLIRoutesSub CLIRoutes

cli :: Parser CLI
cli = CLIRoutesSub <$> (
  (flag' CLIR_list
          ( long "list"
         <> help "List all the available routes" ))
      <|> (CLIR_export <$> strOption
          ( long "export"
         <> metavar "output.json"
         <> help "Export the routes to a file." ))
  )

main :: IO ()
main = runCLI =<< execParser opts
  where
    opts = info (cli <**> helper)
      ( fullDesc
     <> progDesc "Print and exports routes for gargantext"
     <> header "garg-routes helper" )

instance HasRoutes api => HasRoutes (Servant.Auth xs a :> api) where
  getRoutes = getRoutes @api

instance HasRoutes Raw where
  getRoutes = []

runCLI :: CLI -> IO ()
runCLI = \case
  CLIRoutesSub CLIR_list
    -> printRoutes @(NamedRoutes API)
  CLIRoutesSub (CLIR_export filePath)
    -> B.writeFile filePath . BL.toStrict $ encodePretty (getRoutes @(NamedRoutes API))
